"""
DwfInterface.Enumerate - Device enumeration tool for Digilent Waveforms SDK

This module provides utilities for detecting and troubleshooting Digilent hardware devices.
It can be run as a standalone script to list all connected Digilent devices.

When imported as part of the DwfInterface package, it helps with device detection and enumeration.

Usage as standalone script:
    python -m DwfInterface.Enumerate
"""

from ctypes import *
import sys
import time

if sys.platform.startswith("win"):
    dwf = cdll.dwf
    dmgr = cdll.dmgr
    ftd = windll.ftd2xx
elif sys.platform.startswith("darwin"):
    dwf = cdll.LoadLibrary("/Library/Frameworks/dwf.framework/dwf")
    dmgr = cdll.LoadLibrary("/Library/Frameworks/dwf.framework/Frameworks/libdmgr.dylib")
    ftd = cdll.LoadLibrary("/Library/Frameworks/dwf.framework/Frameworks/libftd2xx.dylib")
    dftd = cdll.LoadLibrary("/Library/Frameworks/dwf.framework/Frameworks/libdftd2xx.dylib")
else:
    dwf = cdll.LoadLibrary("libdwf.so")
    dmgr = cdll.LoadLibrary("libdmgr.so")
    ftd = cdll.LoadLibrary("libftd2xx.so")

version = create_string_buffer(32)
cDev = c_int();
dvc = (c_char*1024)()
pdid = c_int()
flags = c_int()
locid = c_int()
type = c_int()

name = create_string_buffer(64)
sn = create_string_buffer(64)

if sys.platform.startswith("darwin"):
    print("")
    if dftd.DFT_Init() == 0 :
        print("DFT_INIT failed")
        quit()

    print("Digilent FTDI Enumeration library loaded")
    
    if dftd.DFT_CreateDeviceInfoList(byref(cDev)) != 0 :
        print("DFT_CreateDeviceInfoList failed")
        dftd.DFT_Term()
        quit()

    print("Devices: "+str(cDev.value))

    for i in range(0, cDev.value):
        if dftd.DFT_GetDeviceInfoDetail(c_int(i), byref(flags), byref(type), byref(pdid), byref(locid), sn, name, None) != 0 :
            print("Failed DFT_GetDeviceInfoDetail")
            dftd.DFT_Term()
            quit()
        print(" "+str(i+1)+". SN:"+str(sn.value)+" '"+str(name.value)+"'"+" flags: "+hex(flags.value)+" type: "+hex(type.value)+" id: "+hex(pdid.value)+" locid: "+hex(locid.value))


print("")
ftd.FT_GetLibraryVersion(byref(pdid))
print("FTDI Version: "+hex(pdid.value))

if ftd.FT_CreateDeviceInfoList(byref(cDev)) != 0 :
    print("FT_CreateDeviceInfoList failed")
    quit()

print("Devices: "+str(cDev.value))

for i in range(cDev.value):
    if ftd.FT_GetDeviceInfoDetail(c_int(i), byref(flags), byref(type), byref(pdid), byref(locid), sn, name, None) != 0 :
        print("Failed FT_GetDeviceInfoDetail")
    print(" "+str(i+1)+". SN:"+str(sn.value)+" "+str(name.value)+""+" flags: "+hex(flags.value)+" type: "+hex(type.value)+" id: "+hex(pdid.value)+" locid: "+hex(locid.value))


print("")
dmgr.DmgrGetVersion(version)
print("DMGR Version: "+str(version.value))

if dmgr.DmgrEnumDevices(byref(cDev)) == 0 :
    print("DmgrEnumDevices failed")
    quit()

print("Devices: "+str(cDev.value))

for i in range(cDev.value):
    dmgr.DmgrGetDvc(c_int(i), dvc);

    if dmgr.DmgrGetInfo(dvc, 3, name) == 0 : #dinfoProdName
        print("Failed DmgrGetInfo dinfoProdName")
    if dmgr.DmgrGetInfo(dvc, 4, byref(pdid)) == 0 : #dinfoPDID
        print("Failed DmgrGetInfo dinfoPDID")
    if dmgr.DmgrGetInfo(dvc, 5, sn) == 0 : #dinfoSN
        print("Failed DmgrGetInfo dinfoSN")
    print(" "+str(i+1)+". "+str(sn.value)+" "+str(name.value)+""+" PDID: "+hex(pdid.value))


print("")
dwf.FDwfGetVersion(version)
print("DWF Version: "+str(version.value))

if dwf.FDwfEnum(c_int(0), byref(cDev)) == 0 :
    print("DmgrEnumDevices failed")
    quit()

print("Devices: "+str(cDev.value))

for i in range(cDev.value):
    dwf.FDwfEnumDeviceName(c_int(i), name)
    dwf.FDwfEnumSN(c_int(i), sn)
    print(" "+str(i+1)+". "+str(sn.value)+" "+str(name.value)+"")

dmgr.DmgrFreeDvcEnum()
if sys.platform.startswith("darwin"):
    dftd.DFT_Term()


"""
   DWFConstants (definitions file for DWF library)
   Author:  Digilent, Inc.
   Revision:  2024-07-24

   Must install:                       
       Python 2.7 or 3
"""


from ctypes import *

# device handle
#HDWF
hdwfNone = c_int(0)

# device enumeration filters
enumfilterAll        = c_int(0)

enumfilterType     = c_int(0x8000000)
enumfilterUSB      = c_int(0x0000001)
enumfilterNetwork  = c_int(0x0000002)
enumfilterAXI      = c_int(0x0000004)
enumfilterRemote   = c_int(0x1000000)
enumfilterAudio    = c_int(0x2000000)
enumfilterDemo     = c_int(0x4000000)

# device ID
devidEExplorer   = c_int(1)
devidDiscovery   = c_int(2)
devidDiscovery2  = c_int(3)
devidDDiscovery  = c_int(4)
devidADP3X50     = c_int(6)
devidADP5250     = c_int(8)
devidDPS3340     = c_int(9)
devidDiscovery3  = c_int(10)
devidADP5470     = c_int(11)
devidADP5490     = c_int(12)
devidADP2230     = c_int(14)

# device version
devverEExplorerC   = c_int(2)
devverEExplorerE   = c_int(4)
devverEExplorerF   = c_int(5)
devverDiscoveryA   = c_int(1)
devverDiscoveryB   = c_int(2)
devverDiscoveryC   = c_int(3)

# trigger source
trigsrcNone                 = c_ubyte(0)
trigsrcPC                   = c_ubyte(1)
trigsrcDetectorAnalogIn     = c_ubyte(2)
trigsrcDetectorDigitalIn    = c_ubyte(3)
trigsrcAnalogIn             = c_ubyte(4)
trigsrcDigitalIn            = c_ubyte(5)
trigsrcDigitalOut           = c_ubyte(6)
trigsrcAnalogOut1           = c_ubyte(7)
trigsrcAnalogOut2           = c_ubyte(8)
trigsrcAnalogOut3           = c_ubyte(9)
trigsrcAnalogOut4           = c_ubyte(10)
trigsrcExternal1            = c_ubyte(11)
trigsrcExternal2            = c_ubyte(12)
trigsrcExternal3            = c_ubyte(13)
trigsrcExternal4            = c_ubyte(14)
trigsrcHigh                 = c_ubyte(15)
trigsrcLow                  = c_ubyte(16)
trigsrcClock                = c_ubyte(17)

# instrument states
DwfStateReady        = c_ubyte(0)
DwfStateConfig       = c_ubyte(4)
DwfStatePrefill      = c_ubyte(5)
DwfStateArmed        = c_ubyte(1)
DwfStateWait         = c_ubyte(7)
DwfStateTriggered    = c_ubyte(3)
DwfStateRunning      = c_ubyte(3)
DwfStateNotDone      = c_ubyte(6)
DwfStateDone         = c_ubyte(2)

# DwfEnumConfigInfo
DECIAnalogInChannelCount = c_int(1)
DECIAnalogOutChannelCount = c_int(2)
DECIAnalogIOChannelCount = c_int(3)
DECIDigitalInChannelCount = c_int(4)
DECIDigitalOutChannelCount = c_int(5)
DECIDigitalIOChannelCount = c_int(6)
DECIAnalogInBufferSize = c_int(7)
DECIAnalogOutBufferSize = c_int(8)
DECIDigitalInBufferSize = c_int(9)
DECIDigitalOutBufferSize = c_int(10)

# acquisition modes:
acqmodeSingle       = c_int(0)
acqmodeScanShift    = c_int(1)
acqmodeScanScreen   = c_int(2)
acqmodeRecord       = c_int(3)
acqmodeOvers        = c_int(4)
acqmodeSingle1      = c_int(5)

# analog acquisition filter:
filterDecimate = c_int(0)
filterAverage  = c_int(1)
filterMinMax   = c_int(2)
filterAverageFit = c_int(3)

# analog in trigger mode:
trigtypeEdge         = c_int(0)
trigtypePulse        = c_int(1)
trigtypeTransition   = c_int(2)
trigtypeWindow       = c_int(3)

# trigger slope:
DwfTriggerSlopeRise   = c_int(0)
DwfTriggerSlopeFall   = c_int(1)
DwfTriggerSlopeEither = c_int(2)

# trigger length condition
triglenLess       = c_int(0)
triglenTimeout    = c_int(1)
triglenMore       = c_int(2)

# error codes for the functions:                         
dwfercNoErc                  = c_int(0)     #  No error occurred
dwfercUnknownError           = c_int(1)     #  API waiting on pending API timed out
dwfercApiLockTimeout         = c_int(2)     #  API waiting on pending API timed out
dwfercAlreadyOpened          = c_int(3)     #  Device already opened
dwfercNotSupported           = c_int(4)     #  Device not supported
dwfercInvalidParameter0      = c_int(16)    #  Invalid parameter sent in API call
dwfercInvalidParameter1      = c_int(17)    #  Invalid parameter sent in API call
dwfercInvalidParameter2      = c_int(18)    #  Invalid parameter sent in API call
dwfercInvalidParameter3      = c_int(19)    #  Invalid parameter sent in API call
dwfercInvalidParameter4      = c_int(20)    #  Invalid parameter sent in API call

# analog out signal types
funcDC       = c_ubyte(0)
funcSine     = c_ubyte(1)
funcSquare   = c_ubyte(2)
funcTriangle = c_ubyte(3)
funcRampUp   = c_ubyte(4)
funcRampDown = c_ubyte(5)
funcNoise    = c_ubyte(6)
funcPulse    = c_ubyte(7)
funcTrapezium= c_ubyte(8)
funcSinePower= c_ubyte(9)
funcSineNA   = c_ubyte(10)
funcCustomPattern   = c_ubyte(28)
funcPlayPattern     = c_ubyte(29)
funcCustom   = c_ubyte(30)
funcPlay     = c_ubyte(31)

# analog io channel node types
analogioEnable      = c_ubyte(1)
analogioVoltage     = c_ubyte(2)
analogioCurrent     = c_ubyte(3)
analogioPower       = c_ubyte(4)
analogioTemperature = c_ubyte(5)
analogioDmm         = c_ubyte(6)
analogioRange       = c_ubyte(7)
analogioMeasure     = c_ubyte(8)
analogioTime        = c_ubyte(9)
analogioFrequency   = c_ubyte(10)
analogioResistance  = c_ubyte(11)
analogioSlew        = c_ubyte(12)

DwfDmmResistance     = c_double(1)
DwfDmmContinuity     = c_double(2)
DwfDmmDiode          = c_double(3)
DwfDmmDCVoltage      = c_double(4)
DwfDmmACVoltage      = c_double(5)
DwfDmmDCCurrent      = c_double(6)
DwfDmmACCurrent      = c_double(7)
DwfDmmDCLowCurrent   = c_double(8)
DwfDmmACLowCurrent   = c_double(9)
DwfDmmTemperature    = c_double(10)

AnalogOutNodeCarrier  = c_int(0)
AnalogOutNodeFM       = c_int(1)
AnalogOutNodeAM       = c_int(2)

DwfAnalogOutIdleDisable  = c_int(0)
DwfAnalogOutIdleOffset   = c_int(1)
DwfAnalogOutIdleInitial  = c_int(2)
DwfAnalogOutIdleHold     = c_int(3)

DwfDigitalInClockSourceInternal = c_int(0)
DwfDigitalInClockSourceExternal = c_int(1)

DwfDigitalInSampleModeSimple   = c_int(0)
# alternate samples: noise|sample|noise|sample|...  
# where noise is more than 1 transition between 2 samples
DwfDigitalInSampleModeNoise    = c_int(1)

DwfDigitalOutOutputPushPull   = c_int(0)
DwfDigitalOutOutputOpenDrain  = c_int(1)
DwfDigitalOutOutputOpenSource = c_int(2)
DwfDigitalOutOutputThreeState = c_int(3) 

DwfDigitalOutTypePulse      = c_int(0)
DwfDigitalOutTypeCustom     = c_int(1)
DwfDigitalOutTypeRandom     = c_int(2)
DwfDigitalOutTypeROM        = c_int(3)
DwfDigitalOutTypeState      = c_int(4)
DwfDigitalOutTypePlay       = c_int(5)

DwfDigitalOutIdleInit     = c_int(0)
DwfDigitalOutIdleLow      = c_int(1)
DwfDigitalOutIdleHigh     = c_int(2)
DwfDigitalOutIdleZet      = c_int(3)

DwfAnalogImpedanceImpedance         = c_int(0)
DwfAnalogImpedanceImpedancePhase    = c_int(1)
DwfAnalogImpedanceResistance        = c_int(2)
DwfAnalogImpedanceReactance         = c_int(3)
DwfAnalogImpedanceAdmittance        = c_int(4)
DwfAnalogImpedanceAdmittancePhase   = c_int(5)
DwfAnalogImpedanceConductance       = c_int(6)
DwfAnalogImpedanceSusceptance       = c_int(7)
DwfAnalogImpedanceSeriesCapacitance = c_int(8)
DwfAnalogImpedanceParallelCapacitance = c_int(9)
DwfAnalogImpedanceSeriesInductance  = c_int(10)
DwfAnalogImpedanceParallelInductance = c_int(11)
DwfAnalogImpedanceDissipation       = c_int(12)
DwfAnalogImpedanceQuality           = c_int(13)
DwfAnalogImpedanceVrms              = c_int(14)
DwfAnalogImpedanceVreal             = c_int(15)
DwfAnalogImpedanceVimag             = c_int(16)
DwfAnalogImpedanceIrms              = c_int(17)
DwfAnalogImpedanceIreal             = c_int(18)
DwfAnalogImpedanceIimag             = c_int(19)

DwfParamUsbPower        = c_int(2) # 1 keep the USB power enabled even when AUX is connected, Analog Discovery 2
DwfParamLedBrightness   = c_int(3) # LED brightness 0 ... 100%, Digital Discovery
DwfParamOnClose         = c_int(4) # 0 continue, 1 stop, 2 shutdown
DwfParamAudioOut        = c_int(5) # 0 disable / 1 enable audio output, Analog Discovery 1, 2
DwfParamUsbLimit        = c_int(6) # 0..1000 mA USB power limit, -1 no limit, Analog Discovery 1, 2
DwfParamAnalogOut       = c_int(7) # 0 disable / 1 enable
DwfParamFrequency       = c_int(8) # Hz
DwfParamExtFreq         = c_int(9) # Hz
DwfParamClockMode       = c_int(10) # 0 internal, 1 output, 2 input, 3 IO
DwfParamTempLimit       = c_int(11) # shutdown temperature
DwfParamFreqPhase       = c_int(12) # system clock phase in 360deg, relative to reference
DwfParamDigitalVoltage  = c_int(13) # digital IO voltage (mV)
DwfParamFreqPhaseSteps  = c_int(14) # number of phase steps, read-only


DwfWindowRectangular    = c_int(0)
DwfWindowTriangular     = c_int(1)
DwfWindowHamming        = c_int(2)
DwfWindowHann           = c_int(3)
DwfWindowCosine         = c_int(4)
DwfWindowBlackmanHarris = c_int(5)
DwfWindowFlatTop        = c_int(6)
DwfWindowKaiser         = c_int(7)
DwfWindowBlackman       = c_int(8)
DwfWindowFlatTopM       = c_int(9)

DwfAnalogCouplingDC     = c_int(0)
DwfAnalogCouplingAC     = c_int(1)

DwfFiirWindow         = c_int(0)
DwfFiirFir            = c_int(1)
DwfFiirIirButterworth = c_int(2)
DwfFiirIirChebyshev   = c_int(3)

DwfFiirLowPass    = c_int(0)
DwfFiirHighPass   = c_int(1)
DwfFiirBandPass   = c_int(2)
DwfFiirBandStop   = c_int(3)

DwfFiirRaw      = c_int(0)
DwfFiirDecimate = c_int(1)
DwfFiirAverage  = c_int(2)


# obsolete
#STS
stsRdy      = c_ubyte(0)
stsArm      = c_ubyte(1)
stsDone     = c_ubyte(2)
stsTrig     = c_ubyte(3)
stsCfg      = c_ubyte(4)
stsPrefill  = c_ubyte(5)
stsNotDone  = c_ubyte(6)
stsTrigDly  = c_ubyte(7)
stsError    = c_ubyte(8)
stsBusy     = c_ubyte(9)
stsStop     = c_ubyte(10)

#TRIGCOND
trigcondRisingPositive   = c_int(0)
trigcondFallingNegative  = c_int(1)

#use device id
enumfilterEExplorer  = c_int(1)
enumfilterDiscovery  = c_int(2)
enumfilterDiscovery2 = c_int(3)
enumfilterDDiscovery = c_int(4)





#imports
import sys
from PyQt5.QtWidgets import QApplication, QMainWindow, QVBoxLayout, QHBoxLayout, QWidget, QLabel, QPushButton
from PyQt5.QtCore import QTimer
from PyQt5.QtGui import QPen, QColor
import pyqtgraph as pg
from pglive.sources.data_connector import DataConnector
from threading import Thread, Lock
from time import sleep
from math import sin
from pglive.sources.live_plot import LiveLinePlot
from pglive.sources.live_plot_widget import LivePlotWidget
import datetime
import numpy as np


# Global variable to hold QApplication instance
_app = None


def get_qt_app():
    """Returns the global QApplication instance, creating it if necessary."""
    global _app
    if _app is None:
        _app = QApplication.instance()
        if _app is None:
            _app = QApplication(sys.argv)
    return _app


class OscilloscopeUI(QMainWindow):
    """usage:
    scope.ui = OscilloscopeUI(scope)
    then the data connector gets assigned to scope,
    and all data acq will call the data connector
    """
    def __init__(self, scope, app=None, npoints=1000):
        super().__init__()
        # Use provided app or get the global one
        self.ds = int(npoints / 1000)
        self.app = app if app is not None else get_qt_app()
        self.scope = scope
        self.lock = Lock()
        self.setup_ui_elements()
        
        # Configure data connectors with proper settings based on the pglive implementation
        # - update_rate: Controls how frequently new data is accepted (in Hz)
        # - plot_rate: Controls how frequently the plot is actually updated (in Hz)
        # - max_points: Controls maximum number of points stored
        self.data_connectors = [
            DataConnector(self.ch1_plot, update_rate=10, plot_rate=30, ignore_auto_range=True),
            DataConnector(self.ch2_plot, update_rate=10, plot_rate=30, ignore_auto_range=True)
        ]

        # Fix timestamp update issue by directly connecting to signals
        for connector in self.data_connectors:
            # Connect to the signal that's emitted when data is updated
            connector.sig_new_data.connect(lambda *args: self.update_timestamp())

        scope.set_data_connectors(self.data_connectors)
        self.running = False
        self.start()

    def update_Npoints(self, Npoints):
        """Update the number of points and downsampling in the data connectors
        
        This updates:
        1. max_points in the data connectors
        2. downsampling rate in the plot widgets
        3. Refreshes the data visualization
        """
        # Update downsampling rate - higher points = higher ds value
        self.ds = max(1, int(Npoints / 1000))
        
        # Update plot widget downsampling settings
        for plot_widget in [self.plot_widget_ch1, self.plot_widget_ch2]:
            plot_widget.setDownsampling(ds=self.ds, auto=True, mode='mean')
        
        # Store current data to restore after resizing
        with self.lock:
            # Extract current data from the connectors
            data_x = [list(conn.x) for conn in self.data_connectors]
            data_y = [list(conn.y) for conn in self.data_connectors]
            
            # Update the max_points property in the connectors
            for connector in self.data_connectors:
                connector.max_points = Npoints
            
            # Restore data if there was any
            for i, connector in enumerate(self.data_connectors):
                if data_x[i] and data_y[i]:
                    # Use cb_set_data to restore the data with new buffer sizes
                    connector.cb_set_data(data_y[i], data_x[i])

    def update_timestamp(self):
        """Update the timestamp label with the current time"""
        current_time = datetime.datetime.now().strftime("%Y-%m-%d %H:%M:%S.%f")[:-3]
        self.timestamp_label.setText(f"Last Update: {current_time}")

    def setup_ui_elements(self):
        """Setup. two charts with a shared x axis, ch1,ch2"""
        self.setWindowTitle("Diligent Digilent Grapher")
        self.setGeometry(100, 100, 800, 600)
        
        # Create a QVBoxLayout
        layout = QVBoxLayout()
        
        # Add timestamp label at the top
        self.timestamp_label = QLabel("Last Update: --")
        self.timestamp_label.setStyleSheet("font-weight: bold;")
        layout.addWidget(self.timestamp_label)
        
        # Create two plot widgets with pglive
        self.plot_widget_ch1 = LivePlotWidget(title="Channel 1")
        self.plot_widget_ch2 = LivePlotWidget(title="Channel 2")
        
        # Configure plot widgets for smoother data visualization
        for plot_widget in [self.plot_widget_ch1, self.plot_widget_ch2]:
            # Enable anti-aliasing for smoother line rendering
            plot_widget.setAntialiasing(True)
            # Let pyqtgraph handle downsampling automatically
            plot_widget.setDownsampling(ds=self.ds, auto=True, mode='mean')
            # Only render what's visible in the view
            plot_widget.setClipToView(True)
            # Add grid for better readability
            plot_widget.showGrid(x=True, y=True, alpha=0.3)
            # Configure view box
            view_box = plot_widget.getPlotItem().getViewBox()
            # Enable auto-range but optimize for performance
            view_box.enableAutoRange(x=True, y=True)
            view_box.setAutoVisible(y=True)
            # Set proper mouse interaction mode
            view_box.setMouseMode(view_box.RectMode)
            # Improve performance with limited range updates
            #sview_box.setLimits(yMin=-10, yMax=10)
        
        # Create the plots with smoother line rendering
        from PyQt5.QtCore import Qt
        yellow_pen = QPen(QColor(255, 255, 0))
        yellow_pen.setWidth(0)  # Slightly thicker for smoother appearance
        yellow_pen.setStyle(Qt.SolidLine)
        
        blue_pen = QPen(QColor(0, 0, 255))
        blue_pen.setWidth(0)  # Slightly thicker for smoother appearance
        blue_pen.setStyle(Qt.SolidLine)
        
        # Create plots with proper connection mode for smooth lines
        self.ch1_plot = LiveLinePlot(pen=yellow_pen, connect="all")
        self.ch2_plot = LiveLinePlot(pen=blue_pen, connect="all")
        
        # Add each plot to its respective widget
        self.plot_widget_ch1.addItem(self.ch1_plot)
        self.plot_widget_ch2.addItem(self.ch2_plot)
        
        # Link X axes of both plots
        self.plot_widget_ch2.setXLink(self.plot_widget_ch1)
        
        # Add plot widgets to layout vertically
        layout.addWidget(self.plot_widget_ch1)
        layout.addWidget(self.plot_widget_ch2)
        
        # Create a single reset button that resets both views
        reset_button = QPushButton("Reset Views")
        reset_button.setFixedWidth(100)  # Make button small and fixed width
        reset_button.clicked.connect(self.reset_all_views)
        
        # Create a horizontal layout to center the button
        button_layout = QHBoxLayout()
        button_layout.addStretch()
        button_layout.addWidget(reset_button)
        button_layout.addStretch()
        
        # Add button layout below the graphs
        layout.addLayout(button_layout)
        
        # Create central widget to hold the layout
        central_widget = QWidget()
        central_widget.setLayout(layout)
        self.setCentralWidget(central_widget)

    def start(self):
        self.show()  # Show the main window instead of just the plot widget
        self.running = True

    def stop(self):
        with self.lock:
            self.scope.data_connectors = None
            self.running = False
            
    def reset_plots(self):
        """Reset the plots by recreating the data connectors"""
        # Create new data connectors with the same optimized settings
        self.data_connectors = [
            DataConnector(self.ch1_plot, max_points=1000, update_rate=5, plot_rate=30, ignore_auto_range=True),
            DataConnector(self.ch2_plot, max_points=1000, update_rate=5, plot_rate=30, ignore_auto_range=True)
        ]
        
        # Fix timestamp update issue by directly connecting to signals in reset_plots too
        for connector in self.data_connectors:
            # Connect to the signal that's emitted when data is updated
            connector.sig_new_data.connect(lambda *args: self.update_timestamp())
        
        # Reapply the pen configuration for smooth rendering
        from PyQt5.QtCore import Qt
        yellow_pen = QPen(QColor(255, 255, 0))
        yellow_pen.setWidth(0)  # Slightly thicker for smoother appearance
        yellow_pen.setStyle(Qt.SolidLine)
        
        blue_pen = QPen(QColor(0, 0, 255))
        blue_pen.setWidth(0)  # Slightly thicker for smoother appearance
        blue_pen.setStyle(Qt.SolidLine)
        
        self.ch1_plot.setPen(yellow_pen)
        self.ch2_plot.setPen(blue_pen)
        
        # Update the scope with the new connectors
        self.scope.set_data_connectors(self.data_connectors)

    def reset_all_views(self):
        """Reset the view limits for both channels"""
        # Completely reset view for channel 1
        view_box1 = self.plot_widget_ch1.getPlotItem().getViewBox()
        view_box1.enableAutoRange(x=True, y=True)
        view_box1.setAutoVisible(x=True, y=True)
        view_box1.autoRange()  # Force immediate auto-range

        # Completely reset view for channel 2
        view_box2 = self.plot_widget_ch2.getPlotItem().getViewBox()
        view_box2.enableAutoRange(x=True, y=True)
        view_box2.setAutoVisible(x=True, y=True)
        view_box2.autoRange()  # Force immediate auto-range


# Test code to run the UI with simulated data
if __name__ == "__main__":
    class SimulatedScope:
        """Simple class to simulate a scope for testing the UI"""
        def __init__(self):
            self.data_connectors = None
            self.x = 0
            self.timer = QTimer()
            self.timer.timeout.connect(self.generate_data)
            
        def set_data_connectors(self, connectors):
            self.data_connectors = connectors
            # Start generating data when connectors are set
            self.timer.start(50)  # Update more frequently (every 50ms) to test timestamp updating
            
        def generate_data(self):
            if self.data_connectors is None:
                return
                
            # Generate some sample data - sine waves with different frequencies
            x_values = np.linspace(self.x, self.x + 0.1, 10)
            ch1_data = np.sin(2 * np.pi * 1.0 * x_values)
            ch2_data = 0.5 * np.sin(2 * np.pi * 2.0 * x_values + 0.5)
            
            # Push data to both channels
            for i, x in enumerate(x_values):
                if self.data_connectors[0]:
                    self.data_connectors[0].cb_append_data_point(y=ch1_data[i], x=x)
                if self.data_connectors[1]:
                    self.data_connectors[1].cb_append_data_point(y=ch2_data[i], x=x)
            
            self.x += 0.1
    
    # Create the simulated scope
    scope = SimulatedScope()
    
    # Create the UI
    app = get_qt_app()
    ui = OscilloscopeUI(scope)
    
    # Start the Qt event loop
    sys.exit(app.exec_())


